/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.cream.jobmanagement.db.table.InputFileTable;
import org.glite.ce.creamapi.jobmanagement.db.table.InputFileTableInterface;

public class InputFileTableTest {
	String datasourceName = null;
	InputFileTableInterface   inputFileTable   = null;
	
	public InputFileTableTest()throws Exception{
	  this.inputFileTable   = new InputFileTable();
	}
	
	public int executeInsertTest(String jobId) throws DatabaseException{
		String[] inputFile = new String[3];
		inputFile[0] = "inputFile1";
		inputFile[1] = "inputFile2";
		inputFile[2] = "inputFile3";
		int rowInserted = -1;
		Connection connection = null;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowInserted = inputFileTable.executeInsert(jobId, inputFile, connection);
		      connection.commit();
		      System.out.println("rowInserted = " + rowInserted);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  } 
		return rowInserted;
	}

	 public int executeDeleteTest(String jobId) throws SQLException, DatabaseException {
		 Connection connection = null;
		 int rowDeleted = -1;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowDeleted = inputFileTable.executeDelete(jobId, connection);
		      connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return rowDeleted; 
	 }
	 
	  public static void main(String[] args) throws Exception{
		  InputFileTableTest inputFileTableTest = new InputFileTableTest();
		  
		  String jobId = "CREAM506445737";
		  //TEST INSERT
		  int rowInserted = inputFileTableTest.executeInsertTest(jobId);
		  System.out.println("rowInserted must be 3 ... = " + rowInserted);
		  //TEST delete
		  int rowDeleted = inputFileTableTest.executeDeleteTest(jobId);
		  System.out.println("rowDeleted must be 3 ... = " + rowDeleted);
	  }
}

