/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.cream.jobmanagement.db.table.JobStatusTable;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.db.table.JobStatusTableInterface;

public class JobStatusTableTest {
	String datasourceName = null;
	JobStatusTableInterface   jobStatusTable   = null;
	
	public JobStatusTableTest()throws Exception{
	  this.jobStatusTable   = new JobStatusTable();
	}
	
	  public void executeInsertTest(String jobId) throws DatabaseException{
		  int rowInserted, totRowInserted = 0;
		  JobStatus jobStatus = null;
		  List<JobStatus> jobStatusList = new ArrayList<JobStatus>();
		  
		  for (int index = 0; index<9; index++){
			  jobStatus = new JobStatus(index);
			  jobStatus.setJobId(jobId);
			  jobStatus.setExitCode("" + index);
			  jobStatusList.add(jobStatus);
		  }
		  Connection connection = null;
		  try{
		    connection = DatasourceManager.getConnection(this.datasourceName);
			rowInserted = jobStatusTable.executeInsert(jobStatusList, connection);
		    totRowInserted += rowInserted;
  		    connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  } 
		  System.out.println("TotRowInserted = " + totRowInserted);
	  }
	  
	 public JobStatus executeSelectLastJobStatusTest(String jobId) throws DatabaseException{
		 Connection connection = null;
		 JobStatus jobStatus = null;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
			  jobStatus = jobStatusTable.executeSelectLastJobStatus(jobId, connection);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return jobStatus;
	  }
	 
	 public List<JobStatus> executeSelectJobStatusHistoryTest(String jobId) throws DatabaseException {
		 Connection connection = null;
		 List<JobStatus> jobStatusList = null;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
			  jobStatusList =  jobStatusTable.executeSelectJobStatusHistory(jobId, connection);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		  return jobStatusList;
	  }
	 
	 public int executeDeleteTest(String jobId) throws DatabaseException{
		 Connection connection = null;
		 int rowDeleted = -1;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowDeleted = jobStatusTable.executeDelete(jobId, connection);
		      connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		 return rowDeleted;
	 }
	 
	 
	  public static void main(String[] args) throws Exception{
		  JobStatusTableTest jobStatusTableTest = new JobStatusTableTest();
		  JobStatus jobStatus = null;
		  String jobId = "CREAM506445737";
		  //TEST INSERT
		  jobStatusTableTest.executeInsertTest(jobId);
		  //TEST executeSelectLastJobStatus
		  jobStatus = jobStatusTableTest.executeSelectLastJobStatusTest(jobId);
		  System.out.println("Last type must be 8 ... = " + jobStatus.getType());
	      //TEST executeSelectJobStatusHistory
		  List<JobStatus> jobStatusList = jobStatusTableTest.executeSelectJobStatusHistoryTest(jobId);
		  for (int i=0; i<jobStatusList.size(); i++){
			  jobStatus = jobStatusList.get(i);
			  System.out.println("JobStatus type = " + jobStatus.getType());
		  }
		  //TEST delete
		  int rowDeleted = jobStatusTableTest.executeDeleteTest(jobId);
		  System.out.println("rowDeleted must be 9 ... = " + rowDeleted);
	  }
	  
}
